/**
 * 入口函数
 * @param {Object} jsonData
 */
define(function() {
	var oldCinemaQueryView = null;
	var cinemasTaskHandler;

	// 页面默认条目数
	var defaultCinemaItems = 6;
	var cinemasData = {
		"data" : {}
	};
	var cinemasTotalServerPages = 0;
	var cinemasCurrentServerPage = 0;
	// 是否从服务端完成数据获取
	var isObtainCinemaDataComplete = true;

	// 正在上映是否加载下一页数据
	var isNeedLoadNextCinemasPage = false;
	// 等待加载的正在上映页码
	var beLoadCinemasPage = 0;

	var cinemas_loading_update_id = null;
	
	window.cinemasView = window.cinemasView || {};
	cinemasView.cinemasNetError = cinemasNetError;
	cinemasView.updateCinemasNetState = updateCinemasNetState;
	cinemasView.callbackJsWithCinemaData = callbackJsWithCinemaData;
	cinemasView.updateCinemasNetState = updateCinemasNetState;
	cinemasView.cinemasNetworkOpened = cinemasNetworkOpened;

	/**
	 * 创建影院查询界面
	 * @param {Object} data
	 */
	function createMovieCinemaQueryView(data) {
		// 删除老的View
		if (oldCinemaQueryView != null && oldCinemaQueryView.parentNode == document.body) {
			removeChild(document.body, oldCinemaQueryView);
			resetCinemas();
			// 调用java函数释放底层资源
			// exec(*, *, *);
		}
		initCinemas(data);
		var outDiv = createOutDiv();
		if (enableAnimation && isChangingSkin == false) {
			outDiv.className += " css_dialog_opacity_animation";
		}
		var cinema_query_view_container = document.createElement("div");
		cinema_query_view_container.className = "css_cinema_query_view_container_div";
		// 上部分
		var top_part = document.createElement("div");
		top_part.className = "css_cinema_top_part_div";
		var cityCinemaLabel = document.createElement("div");
		cityCinemaLabel.className = "css_city_cinema_div";
		cityCinemaLabel.innerText = data.city + "的影院";

		top_part.appendChild(cityCinemaLabel);

		// 中部分（主体）
		var middle_part = document.createElement("div");
		var cinemaDialog = createCinemasDialogView("id_cinemas_dialog", "id_cinemas_canvas", "id_cinemas_refresh", "id_cinemas_failertext");
		var middle_part_items = createCinemaItems();
		middle_part_items.className = "css_middle_part_items_div";
		middle_part.appendChild(cinemaDialog);
		middle_part.appendChild(middle_part_items);
		initCinemaPageValue(middle_part_items, data.cinemas);

		// 下部分 页面切换
		var bottom_part = createCinemaNavigationBar(data.totalPage);

		cinema_query_view_container.appendChild(top_part);
		cinema_query_view_container.appendChild(middle_part);
		cinema_query_view_container.appendChild(bottom_part);

		outDiv.appendChild(cinema_query_view_container);
		document.body.appendChild(outDiv);
		oldCinemaQueryView = outDiv;

		var cinemaTouchObj = createCinemasHandleTouchObject("css_cinema_item_div", "css_cinema_name_div");
		enableCinemasScrollTouchEvent(middle_part_items, 15, cinemaTouchObj);
		middle_part.addEventListener("scrolltouch", function() {
			exec(cinemasTaskHandler, 'statistic', ["DETAILS_CLICKED"]);
			exec(cinemasTaskHandler, 'launcherCinema', [cinemaTouchObj.returnArgument]);
		}, false);
			exec(cinemasTaskHandler, 'statistic', ["SHOW_DETAIL"]);
		return outDiv;

	}

	function initCinemas(data) {
		cinemasData.data = data;
		cinemasTotalServerPages = data.totalServerPages;
		cinemasCurrentServerPage = data.currentServerPage;
		cinemasTaskHandler = data.taskHandler;
	}

	function resetCinemas() {
		cinemasData.data = "";
		cinemasTotalServerPages = 0;
		cinemasCurrentServerPage = 0;
		isObtainCinemaDataComplete = true;
		cinemasTaskHandler = '';
		isNeedLoadNextCinemasPage = false;
		beLoadCinemasPage = 0;
		cinemas_loading_update_id = null;
	}

	/**
	 * 初始界面内容
	 */
	function initCinemaPageValue(parentView, cinemaValues) {
		var cinemaItems = parentView.getElementsByClassName("css_cinema_item_div");
		for (var i = 0; i < defaultCinemaItems; i++) {
			var itemValue = cinemaValues[i];
			if (itemValue != undefined || itemValue != null) {
				cinemaItems[i].style.visibility = "visible";
				cinemaItems[i].getElementsByClassName("css_cinema_name_div")[0].innerText = itemValue.name;
				cinemaItems[i].getElementsByClassName("css_cinema_name_div")[0].id = itemValue.detail_url;
				cinemaItems[i].getElementsByClassName("css_cinema_distance_label")[0].innerText = itemValue.distance + "公里";
				cinemaItems[i].getElementsByClassName("css_item_body_left_bottom_div")[0].innerText = itemValue.address;
			} else {
				cinemaItems[i].style.visibility = "hidden";
			}
		}
	}

	/**
	 * 创建界面框
	 */
	function createCinemaItems() {
		var items_container = document.createElement("div");
		for (var i = 0; i < defaultCinemaItems; i++) {
			var cinemaItem = document.createElement("div");
			cinemaItem.className = "css_cinema_item_div";
			var item_body = createItemBody();
			item_body.className = "css_item_body_div";
			var item_baseline = document.createElement("div");
			item_baseline.className = "css_cinema_item_baseline_div";

			cinemaItem.appendChild(item_body);
			cinemaItem.appendChild(item_baseline);
			items_container.appendChild(cinemaItem);
		}
		return items_container;
	}

	function createCinemasDialogView(dialogId, canvasId, refreshId, failerTextId) {
		// loading 状态下的显示元素
		var dialog_movies_loading_div = document.createElement("div");
		dialog_movies_loading_div.id = dialogId;
		dialog_movies_loading_div.style.display = "none";
		dialog_movies_loading_div.className = "css_dialog_movies_loading";
		// loading使用canvas模拟
		var dialog_movies_loading_canvas = document.createElement("canvas");
		dialog_movies_loading_canvas.id = canvasId;
		// FIXME 暂时借用dialog_title_canvas_height的值作为canvas的宽、高，后续使用独立的变量
		dialog_movies_loading_canvas.setAttribute("width", dialog_title_canvas_height + "px");
		dialog_movies_loading_canvas.setAttribute("height", dialog_title_canvas_height + "px");
		var refreshImg = document.createElement("img");
		refreshImg.src = dialog_stock_refresh;
		refreshImg.id = refreshId;
		refreshImg.className = "css_dialog_refresh_img";
		refreshImg.addEventListener("touchstart", function() {
			cinemasRefreshEvent(event);
		});
		var dialog_movies_loading_failertext = document.createElement("div");
		dialog_movies_loading_failertext.setAttribute("id", failerTextId);
		dialog_movies_loading_failertext.className = "css_dialog_movies_loading_failertext";
		dialog_movies_loading_failertext.innerText = "网络连接超时，请稍后重试";

		dialog_movies_loading_div.appendChild(dialog_movies_loading_canvas);
		dialog_movies_loading_div.appendChild(refreshImg);
		dialog_movies_loading_div.appendChild(dialog_movies_loading_failertext);

		return dialog_movies_loading_div;
	}
	
	function cinemasRefreshEvent(event) {
		var target = event.target;
		if (target.id == "id_cinemas_refresh") {
			getCinemaPageDataFromServer(parseInt(cinemasCurrentServerPage) + 1, cinemasData.data.dataSourceId, cinemasData.data.dataSourceName);
			isObtainCinemaDataComplete = false;
		}
	}
	/**
	 * 创建 一条记录主体
	 */
	function createItemBody() {
		var itemBody = document.createElement("div");
		var itemBodyLeft = document.createElement("div");
		itemBodyLeft.className = "css_item_body_left_div";

		var itemBodyLeft_top = document.createElement("div");
		itemBodyLeft_top.className = "css_item_body_left_top_div";
		var cinema_name = document.createElement("div");
		cinema_name.className = "css_cinema_name_div";
		var cinema_distance = document.createElement("label");
		cinema_distance.className = "css_cinema_distance_label";
		itemBodyLeft_top.appendChild(cinema_name);
		itemBodyLeft_top.appendChild(cinema_distance);

		var itemBodyLeft_bottom = document.createElement("div");
		itemBodyLeft_bottom.className = "css_item_body_left_bottom_div";

		itemBodyLeft.appendChild(itemBodyLeft_top);
		itemBodyLeft.appendChild(itemBodyLeft_bottom);

		var itemBodyRight = createCinemasLinkImg();
		itemBodyRight.className = "css_item_body_right_img";

		itemBody.appendChild(itemBodyLeft);
		itemBody.appendChild(itemBodyRight);

		return itemBody;
	}

	/**
	 * 创建底部翻页栏
	 */
	function createCinemaNavigationBar(pages) {
		var cinema_content_navigation_bar = document.createElement("div");
		cinema_content_navigation_bar.className = "css_cinema_content_navigation_bar_div";
		var page_up = document.createElement("img");
		page_up.className = "css_cinema_page_up_img";
		page_up.src = movie_broad_query_bigarrow_left_dark;
		var page_indicator = document.createElement("label");
		page_indicator.className = "css_cinema_page_indicator_label";
		page_indicator.innerText = "1/" + pages;

		var page_down = document.createElement("img");
		page_down.className = "css_cinema_page_down_img";
		page_down.src = movie_broad_query_bigarrow_right;
		// 组装
		cinema_content_navigation_bar.appendChild(page_up);
		cinema_content_navigation_bar.appendChild(page_indicator);
		cinema_content_navigation_bar.appendChild(page_down);

		cinema_content_navigation_bar.addEventListener("touchstart", function() {
			var target = event.target;
			if (target.className == "css_cinema_page_up_img" && target.src != movie_broad_query_bigarrow_left_dark) {
				target.src = movie_broad_query_bigarrow_left_clicked;
			} else if (target.className == "css_cinema_page_down_img" && target.src != movie_broad_query_bigarrow_right_dark) {
				target.src = movie_broad_query_bigarrow_right_clicked;
			}
		});
		cinema_content_navigation_bar.addEventListener("touchmove", function() {
			var target = event.target;
			if (target.className == "css_cinema_page_up_img" && target.src != movie_broad_query_bigarrow_left_dark) {
				target.src = movie_broad_query_bigarrow_left;
			} else if (target.className == "css_cinema_page_down_img" && target.src != movie_broad_query_bigarrow_right_dark) {
				target.src = movie_broad_query_bigarrow_right;
			}
		});
		cinema_content_navigation_bar.addEventListener("touchend", function() {
			var target = event.target;
			if (target.className == "css_cinema_page_up_img" && target.src != movie_broad_query_bigarrow_left_dark) {
				target.src = movie_broad_query_bigarrow_left;
			} else if (target.className == "css_cinema_page_down_img" && target.src != movie_broad_query_bigarrow_right_dark) {
				target.src = movie_broad_query_bigarrow_right;
			}
			cinemaPagging(event);
		}, false);
		return cinema_content_navigation_bar;
	}
	
	function createCinemasLinkImg() {
		var linkImg = document.createElement("img");
		linkImg.src = dialog_goto_right_icon;
		return linkImg;
	}
	
	/**
	 * 影院查询翻页事件处理
	 */
	function cinemaPagging(event) {
		var targetElement = event.target;
		var targetClassName = targetElement.className;
		var indicator = document.getElementsByClassName("css_cinema_page_indicator_label")[0];
		var indicatorInfo = indicator.innerText.split("/");
		var currentPage = parseInt(indicatorInfo[0]);
		var totalPages = parseInt(indicatorInfo[1]);
		if (targetClassName == "css_cinema_page_up_img" && currentPage > 1) {
			isNeedLoadNextCinemasPage = false;
			completeStoreCinemasData();
			var previousPageData = getCinemaPageDataFromCache(currentPage - 1);
			setCinemaPageValue(previousPageData);
			exec(cinemasTaskHandler, 'statistic', ["PAGGING_CLICKED"]);
			indicator.innerText = (currentPage - 1) + "/" + totalPages;
			if (currentPage == 2) {
				targetElement.src = movie_broad_query_bigarrow_left_dark;
			} else if (currentPage == totalPages) {
				indicator.parentNode.getElementsByClassName("css_cinema_page_down_img")[0].src = movie_broad_query_bigarrow_right;
			}
		} else if (targetClassName == "css_cinema_page_down_img" && currentPage < totalPages) {
			if (!isObtainCinemaDataComplete && !isCinemasNextPageFull(currentPage + 1)) {
				updateCinemasLoadingStatus(indicator, currentPage + 1, totalPages);
				return;
			}
			
			if (!isCinemasNextPageFull(currentPage + 1) && cinemasCurrentServerPage < cinemasTotalServerPages && isCinemasNetStateOk == "false") {
				updateCinemasLoadingStatus(indicator, currentPage + 1, totalPages);
				getCinemaPageDataFromServer(parseInt(cinemasCurrentServerPage) + 1, cinemasData.data.dataSourceId, cinemasData.data.dataSourceName);
				isObtainCinemaDataComplete = false;
				return;
			}
			
			var nextPageData = getCinemaPageDataFromCache(currentPage + 1);
			if (nextPageData != undefined && nextPageData != null && nextPageData.cinemas.length != 0) {
				setCinemaPageValue(nextPageData);
				exec(cinemasTaskHandler, 'statistic', ["PAGGING_CLICKED"]);
				indicator.innerText = (currentPage + 1) + "/" + totalPages;
				if (currentPage + 1 == totalPages) {
					targetElement.src = movie_broad_query_bigarrow_right_dark;
				} else if (currentPage == 1) {
					indicator.parentNode.getElementsByClassName("css_cinema_page_up_img")[0].src = movie_broad_query_bigarrow_left;
				}
			}
		}
		event.stopPropagation();
	}

	function isCinemasNextPageFull(page) {
		var end = page * defaultCinemaItems;
		var pageLastItem = cinemasData.data.cinemas[end - 1];
		if (pageLastItem == null || pageLastItem == undefined) {
			return false;
		} else {
			return true;
		}
	}
	
	function updateCinemasLoadingStatus(indicator, nextPage, totalPages) {
		if (isNeedLoadNextCinemasPage) {
			return;
		}
		document.getElementsByClassName("css_middle_part_items_div")[0].style.display = "none";
		document.getElementById("id_cinemas_dialog").style.display = "block";
		showCinemasDecription("loading");
		var canvas_target = document.getElementById("id_cinemas_canvas");
		cinemas_loading_update_id = startRotate(canvas_target);
		isNeedLoadNextCinemasPage = true;
		beLoadCinemasPage = nextPage;
		indicator.innerText = nextPage + "/" + totalPages;
		if (nextPage == totalPages) {
			indicator.parentNode.getElementsByClassName("css_cinema_page_down_img")[0].src = movie_broad_query_bigarrow_right_dark;
		}
	}
	
	function cinemasNetworkOpened() {
		document.getElementById("id_cinemas_failertext").innerText = "网络已连接，请刷新重试";
		document.getElementById("id_cinemas_refresh").style.display = "inline-block";
	}

	function showCinemasDecription(cinemas_decription) {
		log("current des:" + cinemas_decription);
		if (cinemas_decription == "loading") {
			document.getElementById("id_cinemas_canvas").style.display = "inline-block";
			document.getElementById("id_cinemas_refresh").style.display = "none";
			document.getElementById("id_cinemas_failertext").style.display = "none";
		} else {
			document.getElementById("id_cinemas_canvas").style.display = "none";
			document.getElementById("id_cinemas_refresh").style.display = "none";
			document.getElementById("id_cinemas_failertext").style.display = "block";
		}
	}
	
	var isCinemasNetStateOk = "true";
	function updateCinemasNetState(isNetOk, cmd) {
		if (isNetOk == "false") {
			if (cmd == "cinemas_near") {
				isObtainCinemaDataComplete = true;
				document.getElementById("id_cinemas_failertext").innerText = "网络未打开，请检查网络设置";
				if (cinemas_loading_update_id != null) {
					showCinemasDecription();
				}
			}
		} else {
			if (cmd == "cinemas_near") {
				// 上一次为网络未打开
				if (isCinemasNetStateOk == "false") {
					showCinemasDecription("loading");
				}
				document.getElementById("id_cinemas_failertext").innerText = "网络连接超时，请稍后重试";
			}
		}
		isCinemasNetStateOk = isNetOk;
	}

	/**
	 * 设置页面内容
	 */
	function setCinemaPageValue(pageValue) {
		var cinemaPage = document.getElementsByClassName("css_middle_part_items_div")[0];
		initCinemaPageValue(cinemaPage, pageValue.cinemas);
	}

	/**
	 * 从缓存中获取指定页面数据
	 */
	function getCinemaPageDataFromCache(page) {
		var pageData;
		var start = (page - 1) * defaultCinemaItems;
		var end = page * defaultCinemaItems;
		pageData = subCinemaDataJson(cinemasData.data, start, end);
		return pageData;
	}

	/**
	 * 截取数据中的一部分
	 */
	function subCinemaDataJson(jsonData, start, end) {
		var subData = {
			"cinemas" : []
		};
		var tempCinemas = jsonData.cinemas;
		var dataSourceId = jsonData.dataSourceId;
		var dataSourceName = jsonData.dataSourceName;
		for (var i = start; i < end; i++) {
			var cinemaItem = tempCinemas[i];
			if (cinemaItem == undefined || cinemaItem == null) {
				continue;
			}
			subData.cinemas[i % defaultCinemaItems] = cinemaItem;
		}
		var nextPageLastItem = tempCinemas[end + 17];
		log("nextPageLastItem == " + nextPageLastItem);
		if (nextPageLastItem == undefined || nextPageLastItem == null) {
			if (isObtainCinemaDataComplete && cinemasCurrentServerPage < cinemasTotalServerPages) {
				getCinemaPageDataFromServer(parseInt(cinemasCurrentServerPage) + 1, dataSourceId, dataSourceName);
				isObtainCinemaDataComplete = false;
			}
		}
		return subData;
	}

	/**
	 * 获取服务端指定页的数据
	 */
	function getCinemaPageDataFromServer(page, sourceId, sourceName) {
		if (!isObtainCinemaDataComplete) {
			return;
		}
		var cmd = "cinemas_near";
		exec(cinemasTaskHandler, 'loadNextCinemasServerPage', [cmd, sourceId, sourceName, page]);
	}

	/**
	 * 二次取数回调函数
	 */
	function callbackJsWithCinemaData(data) {
		var dataJson = eval('(' + data + ')');
		var mockData = {
			"cinemas" : []
		};
		var cinemas = dataJson.data;
		var serverCinemas;
		if (cinemas != undefined && cinemas != null) {
			mockData.currentServerPage = dataJson.data.currentServerPage;
			serverCinemas = dataJson.data.cinemas;
		}
		if (serverCinemas == undefined || serverCinemas == null) {
			log("数据为空！");
			return;
		}
		for (var i = 0; i < serverCinemas.length; i++) {
			mockData.cinemas[i] = serverCinemas[i];
		}
		if (mockData != undefined && mockData != null && mockData.cinemas.length > 0) {
			storeCinemaNextPageData(mockData);
			isObtainCinemaDataComplete = true;
			completeStoreCinemasData();
		}
	}

	function completeStoreCinemasData() {
		stopRotate(cinemas_loading_update_id);
		cinemas_loading_update_id = null;
		document.getElementsByClassName("css_middle_part_items_div")[0].style.display = "block";
		document.getElementById("id_cinemas_dialog").style.display = "none";
		if (isNeedLoadNextCinemasPage) {
			var nextPageData = getCinemaPageDataFromCache(beLoadCinemasPage);
			if (nextPageData != undefined && nextPageData != null && nextPageData.cinemas.length != 0) {
				setCinemaPageValue(nextPageData);
			}
			isNeedLoadNextCinemasPage = false;
		}

	}

	/**
	 * 保存数据
	 * @param {Object} nextPageData
	 */
	function storeCinemaNextPageData(nextPageData) {
		var cinemas = nextPageData.cinemas;
		var storeCinemasData = cinemasData.data.cinemas;
		var length = storeCinemasData.length;
		var nextPageDataLength = cinemas.length;
		for (var i = 0; i < nextPageDataLength; i++) {
			cinemasData.data.cinemas[length + i] = cinemas[i];
		}
		cinemasCurrentServerPage = nextPageData.currentServerPage;
		cinemasData.data.currentServerPage = nextPageData.currentServerPage;
	}

	function cinemasNetError() {
		isCinemasNetStateOk = "false";
		showCinemasDecription();
		isObtainCinemaDataComplete = true;
		document.getElementById("id_cinemas_refresh").style.display = "inline-block";
		document.getElementById("id_cinemas_failertext").innerText = "网络连接超时，请稍后重试";
	}
	
	function showCinemasView(jsonResult) {
		var dataType = jsonResult.queryType;
		var data = jsonResult.data;
		var resultView = createMovieCinemaQueryView(data);
		return resultView;

	}
	
	/**
	 * 生成touch事件处理对象
	 */
	function createCinemasHandleTouchObject(parentElementClassName, returnElementClassName) {
		var touchObj = new Object;
		touchObj.returnArgument = null;
		touchObj.touchElement = null;

		touchObj.touchStart = function(event) {
			this.touchElement = event.target;
			this.touchElement = getParentOfClass(parentElementClassName, this.touchElement);
			// this.touchElement.style.background = "#BCD5EF";
			if (this.touchElement != null && this.touchElement != undefined) {
				this.touchElement.style.background = "url('" + list_item_background_pressed + "')";
			}
		};
		touchObj.touchMove = function() {
			if (this.touchElement !== null) {
				this.touchElement.style.background = "";
			}

		};
		touchObj.touchEnd = function() {
			if (this.touchElement != null) {
				this.touchElement.style.background = "";
				this.returnArgument = this.touchElement.getElementsByClassName(returnElementClassName);
				log("touchElement.toStriing === " + this.returnArgument.toString());
				if (this.returnArgument != undefined && this.returnArgument != null) {
					this.returnArgument = this.returnArgument[0].id;
				}
			}
			this.touchElement = null;
		}

		return touchObj;
	}

	function enableCinemasScrollTouchEvent(target, scrollLength, touchObj) {
		if (target == null || target == undefined) {
			return;
		}
		if (scrollLength == undefined) {
			scrollLength = 15;
		}
		var myEvent = document.createEvent("Event");
		// 事件名称：scrolltouch;是否冒泡:true;是否可取消：true;
		myEvent.initEvent("scrolltouch", true, true);
		var target_start_pageY = 0;
		var target_end_pageY = 0;
		var target_have_move_motion = false;
		target.addEventListener("touchstart", function() {
			target_have_move_motion = false;
			event.stopPropagation();
			touchObj.touchStart(event);
			var touchPoint = event.targetTouches[0];
			target_start_pageY = touchPoint.pageY;
		});
		target.addEventListener("touchmove", function() {
			event.stopPropagation();
			touchObj.touchMove();
			var touchPoint = event.targetTouches[0];
			target_end_pageY = touchPoint.pageY;
			if (!target_have_move_motion) {
				if (Math.abs(target_end_pageY - target_start_pageY) > scrollLength) {
					target_have_move_motion = true;
					touchObj.returnArgument = null;
				}
			}
		});
		target.addEventListener("touchend", function() {
			event.stopPropagation();
			touchObj.touchEnd();
			if (!target_have_move_motion && touchObj.returnArgument != null) {
				target.dispatchEvent(myEvent);
			}
			target_start_pageY = 0;
			target_end_pageY = 0;
			target_have_move_motion = false;
			touchObj.touchElement = null;
			touchObj.returnArgument = null;
		});
	}

	function log(str) {
		console.log(str);
	}

	return showCinemasView;
});
